//----------------------------------------------------------------------------
//  Project Test_At3dx_LoadLibrary
//  MicroMouse Productions
//  Copyright  2005-2021. All Rights Reserved.
//  Author: Wayne Hogue
//
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <stdlib.h>
#include <stdio.h>

#include "Test_At3dx_LoadLibraryApp.h"
#include "at3dx_my_functions.h"
#include "at3ddll.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTest_At3dx_LoadLibraryForm *Test_At3dx_LoadLibraryForm;
//---------------------------------------------------------------------------
__fastcall TTest_At3dx_LoadLibraryForm::TTest_At3dx_LoadLibraryForm(TComponent* Owner)
  : TForm(Owner)
{
  unused_W8057_Flag = 0;

  dllOpen    = 0;
  fileLoaded = 0;

  // Use OpenGL view window
  //
  useOglFlag = 1;

  at3dLibHandle = MY_NULL_PTR;

  memset(inName, 0, sizeof(char) * MAX_PATH);

  // --

  Form_Setup();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LoadLibraryForm::Form_Setup(void)
{
#if defined(_WIN64)
  #ifdef _DEBUG
    const char* cptr = " (x64-Debug)";
  #else
    const char* cptr = " (x64)";
  #endif
#else
  #ifdef _DEBUG
    const char* cptr = " (x32-Debug)";
  #else
    const char* cptr = " (x32)";
  #endif
#endif

  Caption = Caption + cptr;

  Update_Labels();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LoadLibraryForm::FormDestroy(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  // Free all memory used by DLL
  //
  Stop_Library();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LoadLibraryForm::DoLoadLibraryClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Start_Library();
  Update_Labels();
}
//---------------------------------------------------------------------------
void __fastcall TTest_At3dx_LoadLibraryForm::DoStartDllClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Start_DLL();
  Update_Labels();
}
//---------------------------------------------------------------------------
void __fastcall TTest_At3dx_LoadLibraryForm::DoStopDllClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Stop_DLL();
  Update_Labels();
}
//---------------------------------------------------------------------------
void __fastcall TTest_At3dx_LoadLibraryForm::DoFreeLibraryClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Stop_Library();
  Update_Labels();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LoadLibraryForm::ReadObjFileClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Read_3D_File();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LoadLibraryForm::Update_Labels(void)
{
  if (at3dLibHandle)
  {
    LibraryLabel->Caption = "Library Loaded.";
  }
  else
  {
    LibraryLabel->Caption = "Library NOT Loaded.";
  }

  if (dllOpen)
  {
    DllLabel->Caption = "DLL Running.";
  }
  else
  {
    DllLabel->Caption = "DLL Stopped.";
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LoadLibraryForm::Start_Library(void)
{
  if (!at3dLibHandle)
  {
    #if defined(_WIN64)
      at3dLibHandle = LoadLibrary("at3dx(x64).dll");
    #else
      at3dLibHandle = LoadLibrary("at3dx(x32).dll");
    #endif
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LoadLibraryForm::Stop_Library(void)
{
  if (at3dLibHandle)
  {
    if (dllOpen) Stop_DLL();

    if (FreeLibrary(at3dLibHandle) != 0) at3dLibHandle = MY_NULL_PTR;
  }
}
//---------------------------------------------------------------------------


void __fastcall TTest_At3dx_LoadLibraryForm::Start_DLL(void)
{
  if ((at3dLibHandle) && (!dllOpen))
  {
    dllOpen = At3d_Start();

    if (dllOpen)
    {
      int errorFlag = 0;

      // To detect mouse scroll wheel messages the DLL has to know the
      // handle to the main window of the application which
      // receives all the messages from the Windows Operating System
      //
      At3d_SetAppHandle(Handle);

      // Set size of DLL window
      //
      At3d_Size(600, 500);

      // Set position of DLL window
      //
      At3d_Position(2, 2);

      // Make DLL window a child of a window in the application
      //
      At3d_SetParent(ViewPanel->Handle);

      // If the DLL window does not have a parent
      // At3d_SetParent(0);
      // a button will appear on the Windows Taskbar
      // use this function to set the name that will appear
      //
      At3d_SetCaption("DLL (at3d)");

      // Use the OpenGL view window
      //
      UseOpenGL->Checked = (useOglFlag) ? 1 : 0;
      At3d_WriteFlag(LIBFLAG_USE_OPENGL, useOglFlag);

      // Light both sides of the polygons
      //
      At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, 1);

      // Cull back faces of polygons
      //
      At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES,1);

      // Make DLL window visible
      //
      At3d_Show();

      if (errorFlag) Stop_DLL();
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LoadLibraryForm::Stop_DLL(void)
{
  if ((at3dLibHandle) && (dllOpen))
  {
    // free all memory used by the DLL
	  //
    At3d_Stop();
    dllOpen = 0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LoadLibraryForm::Read_3D_File(void)
{
  if (!dllOpen) return;

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
	char dir0[_MAX_DIR];
	char name0[_MAX_FNAME];
	char ext0[_MAX_EXT];

  if (inName[0] != 0)
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  AnsiString filter = "Wavefront";
  filter += "|*";
  filter += ".obj";
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

 	if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;
    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);

    my_strcpy(inName, MAX_PATH, fileName.c_str());

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // read file
    //
    fileLoaded = At3d_Read(INTYPE_WAVEFRONT, inName, 1.0);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LoadLibraryForm::UseOpenGLClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  if (!dllOpen) return;

  useOglFlag = (UseOpenGL->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_USE_OPENGL, useOglFlag);
}
//---------------------------------------------------------------------------

